<?php

namespace Drp\NativeEventApp\GraphQL\Mutations;

use Drp\NativeEventApp\Auth\AuthTokenGenerator;
use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Contracts\MagicLinkSigner;
use Drp\NativeEventApp\Contracts\TokenAuthenticatable;
use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Exceptions\CouldNotSendAuthCode;
use Drp\NativeEventApp\Notifications\SendAuthCode;
use Nexmo\Client\Exception\Exception as NexmoException;
use Swift_RfcComplianceException;
use Swift_SwiftException;
use Throwable;

class AuthRequest
{
    /**
     * @var \Drp\NativeEventApp\Contracts\MagicLinkSigner
     */
    private $linkSigner;

    /**
     * @var \Drp\NativeEventApp\Auth\AuthTokenGenerator
     */
    private $tokenGenerator;

    public function __construct(MagicLinkSigner $linkSigner, AuthTokenGenerator $tokenGenerator)
    {
        $this->linkSigner = $linkSigner;
        $this->tokenGenerator = $tokenGenerator;
    }

    /**
     * @param mixed $rootValue
     * @param array $args
     * @return bool // phpcs:ignore
     * @throws CouldNotSendAuthCode
     */
    public function __invoke($rootValue, array $args): bool // phpcs:ignore
    {
        $this->linkSigner->verify($args['authIdentifier'], $args['expiry'], $args['signature']);

        $user = AuthInfoFinder::retrieve($args['authIdentifier']);

        try {
            $this->checkAuthMethod($args['authMethod'], $user);

            $code = $this->tokenGenerator->generate($args['authIdentifier']);

            $user->notify(new SendAuthCode($args['authMethod'], $code));

            return true;
        } catch (NexmoException $exception) {
            throw CouldNotSendAuthCode::viaSms($exception);
        } catch (Swift_RfcComplianceException $exception) {
            throw CouldNotSendAuthCode::invalidEmail($exception);
        } catch (Swift_SwiftException $exception) {
            throw CouldNotSendAuthCode::viaEmail($exception);
        } catch (Throwable $exception) {
            throw CouldNotSendAuthCode::viaMethod($args['authMethod'], $exception);
        }
    }

    private function checkAuthMethod(AuthMethod $authMethod, TokenAuthenticatable $user): void
    {
        throw_unless(
            $user->contactDetails()->has($authMethod->value),
            CouldNotSendAuthCode::invalidAuthMethod($authMethod)
        );
    }
}
