<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Laravel\Passport\AuthCode;
use Laravel\Passport\Passport;
use Laravel\Passport\Token;
use League\OAuth2\Server\AuthorizationServer;
use Mockery;
use Spatie\Snapshots\MatchesSnapshots;
use Zend\Diactoros\Response as Psr7Response;
use Zend\Diactoros\Response\JsonResponse;
use Zend\Diactoros\ServerRequest;

class TokenTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function throws_renderable_exception(): void
    {
        $query = 'mutation {
              token(authIdentifier:"1", code:"1234", deviceId:"d145effc-5ff2-4d02-a443-4a7b8b27ffb5") {
                accessToken
                refreshToken
                expiresIn
                tokenType
              }
            }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function can_exchange_code_for_token(): void
    {
        config([
            'native-event-app.auth.client_id' => '5432',
            'native-event-app.auth.client_secret' => '1234',
        ]);

        $token = $this->prepareToken();

        $this->mock(AuthorizationServer::class, function (Mockery\MockInterface $mock) use ($token): void {
            $mock->shouldReceive('respondToAccessTokenRequest')
                ->withArgs([
                    Mockery::on(function (ServerRequest $request) {
                        return $request->getParsedBody() === $this->expectedRequestBody();
                    }),
                    Mockery::type(Psr7Response::class),
                ])
                ->andReturn($this->fakeOAuthTokenResponse($token))
                ->once();
        });

        $query = 'mutation {
              token(authIdentifier:"1", code:"9876", deviceId:"d145effc-5ff2-4d02-a443-4a7b8b27ffb5") {
                accessToken
                refreshToken
                expiresIn
                tokenType
              }
            }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));

        $device = Device::firstOrFail();
        $token = Token::firstOrFail();

        $this->assertEquals('d145effc-5ff2-4d02-a443-4a7b8b27ffb5', $device->id);
        $this->assertEquals($token->id, $device->access_token_id);
        $this->assertNull($device->firebase_device_id);
    }

    /** @test */
    public function wont_issue_token_for_expired_request(): void
    {
        $query = 'mutation {
              token(authIdentifier:"1", code:"1234", deviceId:"d145effc-5ff2-4d02-a443-4a7b8b27ffb5") {
                accessToken
                refreshToken
                expiresIn
                tokenType
              }
            }';

        AuthCode::create([
            'id' => 1234,
            'user_id' => 1,
            'client_id' => config('native-event-app.auth.client_id'),
            'revoked' => false,
            'expires_at' => Carbon::parse('-1 day'),
        ]);

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function the_access_token_expiry_time_is_configurable(): void
    {
        Carbon::setTestNow('26-05-2020 13:00');

        // Set access token expiry time to 2 minutes.
        config()->set('native-event-app.auth.access_token_ttl', 'P10S');

        $query = 'mutation {
              token(authIdentifier:"1", code:"1234", deviceId:"d145effc-5ff2-4d02-a443-4a7b8b27ffb5") {
                accessToken
                refreshToken
                expiresIn
                tokenType
              }
            }';

        AuthCode::create([
            'id' => 1234,
            'user_id' => 1,
            'client_id' => config('native-event-app.auth.client_id'),
            'revoked' => false,
            'expires_at' => now()->addMonth(),
        ]);

        $response = $this->graphQlRequest($query);
        $response = json_decode($response, true);

        $this->assertArrayHasKey('expiresIn', $response['data']['token']);
        $this->assertEquals(1, Passport::token()->newQuery()->count());

        $token = Passport::token()->newQuery()->first();

        $this->assertEquals(now()->addSeconds(120), $token->expires_at);
    }

    private function prepareToken(): string
    {
        $token = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIiwianRpIjoiMSJ9.eyJqdGkiOiIxIiwiZXhwIjoxNTg3MzY5NjIyfQ.';
        Token::create([
            'id' => 1,
            'client_id' => config('native-event-app.auth.client_id'),
            'revoked' => 0,
        ]);

        return $token;
    }

    private function expectedRequestBody(): array
    {
        return [
            'auth_identifier' => '1',
            'code' => '9876',
            'device_id' => 'd145effc-5ff2-4d02-a443-4a7b8b27ffb5',
            'client_id' => '5432',
            'client_secret' => '1234',
            'grant_type' => 'authorization_token',
        ];
    }

    private function fakeOAuthTokenResponse(string $token): Psr7Response
    {
        return new JsonResponse([
            'token_type' => 'Bearer',
            'expires_in' => '12345',
            'access_token' => $token,
            'refresh_token' => 'REFRESH TOKEN',
        ]);
    }
}
